proc base_file_name {} {
    global db

    if (![info exists db(user)]) {
	error "User name not defined yet! (db_file_name)"
    }
    return $db(user)_vitae
}

proc tex_file_name {} {return [base_file_name].tex}
proc dvi_file_name {} {return [base_file_name].dvi}
proc ps_file_name {}  {return [base_file_name].ps}
proc eps_file_name {} {return [base_file_name].eps}
proc pdf_file_name {} {return [base_file_name].pdf}
proc gif_file_name {} {return [base_file_name].gif}


proc replace_char {str find replace} {
    set f_len [string len $find]
    set r_len [string len $replace]
    set loc 0
    while {[set posn [string first $find \
                      [string range $str $loc end]]] != -1} {
	set str [join [string range $str 0 [expr {$loc + $posn - 1}]] \
		      $replace \
                      [string range $str 0 [expr {$loc + $posn + $f_len}]]]
        set loc [expr {$loc + $posn + $r_len}]
    }
}
    

proc db_to_tex type {
    global db

    set str $db($type)
    regsub -all {\$} $str {\$}     str
    regsub -all %    $str {\%}     str
    regsub -all \{   $str {\{}     str
    regsub -all \}   $str {\}}     str
    regsub -all <    $str {$<$}    str
    regsub -all >    $str {$>$}    str
    regsub -all #    $str {\#}     str
    regsub -all ~    $str {$\sim$} str

    return $str
}

proc start_mini {f width {pos ""}} {
    if {$pos == "indent"} {
	puts -nonewline $f "\\hspace{1.5in}"
    }
    puts $f "\\begin{minipage}\[t\]{$width}"
    switch -exact -- $pos {
	center {puts $f "\\begin{center}"}
	right  {puts $f "\\flushright"}
    }
}

proc end_mini {f {space ""} {pos ""}} {
    if {$pos == "center"} {
	puts $f "\\end{center}"
    }
    if {$space == ""} {
	puts $f "\\end{minipage}"
    } else {
	puts $f "\\end{minipage}\\hspace{$space}%"
    }
}

proc cv_label {f txt} {
    global db

    start_mini $f 1.3in right
    puts $f "{\\bf $txt}"
    end_mini $f .2in
}

proc cv_text {f txt} {
    global db

    start_mini $f 5in
    puts $f $txt
    end_mini $f
    spacer $f
}

proc spacer f {
    puts $f "\n\\skipline\n"
}

proc elec_style f {
    global db

    start_mini $f 2.3in center
    foreach ent {email home_url proj_url} {
	puts $f "$db(.contact.$ent)\n"
    }
    end_mini $f .1in center
}

proc two_address_style f {
    global db

    set c .contact
    set has_elec [expr {$db($c.email) != "" || $db($c.home_url) != "" ||
                        $db($c.proj_url) != ""}]
    if {$has_elec} {
	set width 2.0in
    } {
	set width 3.2in
    }
    start_mini $f $width
    foreach ent {addr1 addr2 addr3 phone} {
	puts $f "$db($c.home_$ent)\n"
    }
    end_mini $f .1in
    if {$has_elec} {
	elec_style $f
    }
    start_mini $f $width right
    foreach ent {addr1 addr2 addr3 phone} {
	puts $f "$db($c.work_$ent)\n"
    }
    if {$db($c.work_fax) != ""} {
	puts $f "FAX: $db($c.work_fax)\n"
    }
    end_mini $f
}

proc one_address_style {f type} {
    global db

    set c .contact
    set has_elec [expr {$db($c.email) != "" || $db($c.home_url) != "" ||
                        $db($c.proj_url) != ""}]
    if {$has_elec} {
	set width 2.0in
    } {
	set width 3.2in
    }
    start_mini $f $width
    foreach ent {addr1 addr2 addr3} {
	puts $f "$db($c.${type}_$ent)\n"
    }
    end_mini $f .1in
    if {$has_elec} {
	elec_style $f
    }
    start_mini $f $width right
    if {$db($c.home_phone) != ""} {
	if {$db($c.work_phone) != ""} {
	    puts $f "Home: $db($c.home_phone)\n"
	    puts $f "Office: $db($c.work_phone)\n"
	} {
	    if {$type != "home"} {
		puts -nonewline $f "Home: "
	    }
	    puts $f "$db($c.home_phone)\n"
	}
    } {
	if {$db($c.work_phone) != ""} {
	    if {$type != "work"} {
		puts -nonewline $f "Office: "
	    }
	    puts $f "$db($c.work_phone)\n"
	}
    }
    if {$db($c.work_fax) != ""} {
	puts $f "FAX: $db($c.work_fax)\n"
    }
    end_mini $f
}

proc zero_address_style f {
    global db

    set c .contact
    set has_elec [expr {$db($c.email) != "" || $db($c.home_url) != "" ||
                        $db($c.proj_url) != ""}]
    start_mini $f 6.5in center
    if {$has_elec} {
	# balance space on right side of elec_style output
	puts $f "\\hspace{.1in}%"
	elec_style $f
	puts $f "\n"
    }
    if {$db($c.home_phone) != ""} {
	if {$db($c.work_phone) != ""} {
	    puts $f "Home: $db($c.home_phone)\n"
	    puts $f "Office: $db($c.work_phone)\n"
	} {
	    puts $f "$db($c.home_phone)\n"
	}
    } {
	if {$db($c.work_phone) != ""} {
	    puts $f "$db($c.work_phone)\n"
	}
    }
    if {$db($c.work_fax) != ""} {
	puts $f "FAX: $db($c.work_fax)\n"
    }
    end_mini $f "" center
}

proc write_contact f {
    global db

    set c .contact
    puts $f "\\centerline{\\LARGE $db($c.name)}\n"
    puts $f "\\vspace{5pt}\\hrule\\vspace{5pt}\n"
    set work_addr [expr {$db($c.work_addr1) != "" ||
                         $db($c.work_addr2) != "" ||
                         $db($c.work_addr3) != ""}]
    if {$db($c.home_addr1) != "" || $db($c.home_addr2) != "" ||
	$db($c.home_addr3) != ""} {
	if {$work_addr} {
	    two_address_style $f
	} {
	    one_address_style $f home
	}
    } {
	if {$work_addr} {
	    one_address_style $f work
	} {
	    zero_address_style $f
	}
    }
	
    puts $f "\n\\vspace{5pt}\\hrule\\vspace{10pt}\n"
}

proc write_objective f {
    global db
    
    set c .objective
    foreach type {obj res_int tch_int} {
	if {$db($c.${type}_text) != ""} {
	    cv_label $f $db($c.${type}_0)
	    cv_text $f $db($c.${type}_text)
	}
    }
}

proc write_degree {f i} {
    global db

    set c .degree${i}
    if {$db($c.loc) != ""} {
	puts $f "$db($c.inst), $db($c.loc)\n"
    } {
	puts $f "$db($c.inst)\n"
    }
    puts $f "$db($c.deg) in $db($c.fld) $db($c.stat.a) $db($c.date)."
    if {$db($c.adv) != ""} {
	puts $f "  Advisor: $db($c.adv)\n"
    } {
	puts $f ""
    }
    if {$db($c.tit) != ""} {
	puts $f "Thesis: ``$db($c.tit).''"
    }
    puts $f "  $db($c.desc)"
    if {$db($c.gpa) != ""} {
	puts $f "  GPA: $db($c.gpa)."
    }
    if {$db($c.rank) != ""} {
	puts $f "  Rank: $db($c.rank)."
    }
}

proc write_education f {
    global db

    set c .education
    if {$db($c.deglist) == ""} {
	return
    }
    cv_label $f Education
    start_mini $f 5in
    write_degree $f [lindex $db($c.deglist) 0]
    end_mini $f
    spacer $f

    foreach i [lrange $db($c.deglist) 1 end] {
	start_mini $f 5in indent
	write_degree $f $i
	end_mini $f
	spacer $f
    }
}

proc write_award f {
    global db
    
    set c .award
    foreach type {award res_skl tch_skl} {
	if {$db($c.${type}_text) != ""} {
	    cv_label $f $db($c.${type}_0)
	    cv_text $f $db($c.${type}_text)
	}
    }
}

proc write_job {f c} {
    global db

    puts -nonewline $f $db($c.inst)
    if {$db($c.loc) != ""} {
	puts -nonewline $f ", $db($c.loc)"
    }
    if {$db($c.date) != ""} {
	puts $f ", {\\em $db($c.date)}\n"
    } {
	puts $f "\n"
    }
    puts $f "$db($c.job)."
    puts $f $db($c.desc)
}

proc write_experience f {
    global db

    set c .experience
    if {$db($c.joblist) == ""} {
	return
    }
    cv_label $f $db($c.exp_0)
    start_mini $f 5in
    write_job $f .job[lindex $db($c.joblist) 0]
    end_mini $f
    spacer $f

    foreach i [lrange $db($c.joblist) 1 end] {
	start_mini $f 5in indent
	write_job $f .job${i}
	end_mini $f
	spacer $f
    }
}

proc write_teachex f {
    global db

    set c .teachex
    if {$db($c.texlist) == ""} {
	return
    }
    cv_label $f $db($c.exp_0)
    start_mini $f 5in
    write_job $f .tex[lindex $db($c.texlist) 0]
    end_mini $f
    spacer $f

    foreach i [lrange $db($c.texlist) 1 end] {
	start_mini $f 5in indent
	write_job $f .tex${i}
	end_mini $f
	spacer $f
    }
}

proc write_activities f {
    global db

    set c .activities
    if {$db($c.text) != ""} {
	cv_label $f $db($c.act_0)
	cv_text $f $db($c.text)
    }
}

proc write_dissertation f {
    global db
    
    set c .dissertation
    if {$db($c.text) != ""} {
	cv_label $f $db($c.dis_0)
	start_mini $f 5in
	if {$db($c.title) != ""} {
	    puts $f "\\begin{center}{\\em $db($c.title)}\\end{center}\n"
	}
        puts $f $db($c.text)
	end_mini $f
        spacer $f
    }
}

proc write_publication {f num idx} {
    global db

    set c .publication${num}_${idx}
    set str $db($c.desc)

#    while {[regsub \" $str `` str] > 0} {
#	regsub \" $str '' str
#    }
#    while {[regsub < $str "\{\\em " tmp] > 0} {
#	if {[regsub > $tmp "\}" tmp] == 1} {set str $tmp}
#    }

    puts $f $str
}

proc write_publications f {
    global db

    foreach num $db(.publications.pubtypelist) {
	set c .publication_type${num}
	if {$db($c.publist) == ""} {
	    continue
	}
	cv_label $f $db($c.type)
	start_mini $f 5in
	write_publication $f $num [lindex $db($c.publist) 0]
	end_mini $f
	spacer $f
	foreach i [lrange $db($c.publist) 1 end] {
	    start_mini $f 5in indent
	    write_publication $f $num $i
	    end_mini $f
	    spacer $f
	}
    }
}

proc write_reference {f i} {
    global db

    foreach t {name posn inst addr1 addr2 addr3 addr4 phone email} {
	puts $f "$db(.reference${i}.${t})\n"
    }
}

proc write_references f {
    global db

    set c .references
    if {$db($c.reflist) == ""} {
	return
    }
    cv_label $f References
    start_mini $f 2.4in
    write_reference $f [lindex $db($c.reflist) 0]

    if {[llength $db($c.reflist)] > 1} {
	end_mini $f .2in
	start_mini $f 2.4in
	write_reference $f [lindex $db($c.reflist) 1]
    }
    end_mini $f
    spacer $f

    set is_left 1
    foreach i [lrange $db($c.reflist) 2 end] {
	if {$is_left} {
	    start_mini $f 2.4in indent
	    write_reference $f $i
	    end_mini $f .2in
	    set is_left 0
	} {
	    start_mini $f 2.4in
	    write_reference $f $i
	    end_mini $f
	    spacer $f
	    set is_left 1
	}
    }
    if {!$is_left} {
	spacer $f
    }
}

proc write_resume f {
    global db

    write_contact $f
    write_objective $f
    write_education $f
    write_award $f
    write_experience $f
    write_teachex $f
    write_activities $f
    write_dissertation $f
    write_publications $f
    write_references $f
}


proc create_resume {} {
    global path ctrl

    set tex_name [tex_file_name]
    set dvi_name [dvi_file_name]
    set ps_name  [ps_file_name]
    set pdf_name [pdf_file_name]

    if {$ctrl(use_gui)} {
	switch_windows .status
	tkwait visibility .status
    }

    set_status "Building LaTeX file..." ""
    if {[catch {
	file copy -force $path(vitae_header) $tex_name
	set res_file [open $tex_name a]
	write_resume $res_file
	puts $res_file "\\end{document}"
	close $res_file
    } msg]} {
	catch {close $res_file}
	set_status "Building LaTeX file..." "Failed: $msg"
	return
    }
    set_status "Building LaTeX file..." "Done."

    if {$ctrl(target) != "tex"} {
        if {![issue_command "$path(latex) $tex_name" "Latex'ing vitae..."]} {
	    return
	}
	if {$ctrl(target) != "dvi" &&
            ![issue_command "$path(dvips) -o $ps_name $dvi_name" \
		    "Creating PostScript..."]} {
	    return
	}
	if {$ctrl(target) == "pdf" &&
            ![issue_command "$path(ps2pdf) $ps_name" "Creating PDF..."]} {
	    return
	}
    }
    switch -exact $ctrl(target) {
	dvi {set out $dvi_name}
	pdf {set out $pdf_name}
	ps  {set out $ps_name}
	tex {set out $tex_name}
	default {error "unknown target type"}
    }
    set_status "Copying to output file..." ""
    if {[catch {file rename -force $out $path(out)} msg]} {
	set_status "Copying to output file..." "Failed: $msg"
	return
    }
    set_status "Vitae generation is complete." ""
}
