#
# global variable db holds all database information
#
# user         login name
# others as specified by windows in var_list
#

proc init_files {} {
    global db path ctrl

    set db(reading) 0
    if {[catch {set db(user) [exec whoami]} msg]} {
	puts "Could not determine user name: $msg"
	exit
    }
    set path(tmp) /tmp/vitae-$db(user)
    if {[catch {file mkdir $path(tmp)} msg]} {
	puts "WARNING: using current directory for temporary files."
	set path(tmp) .
    }
    if {[catch {cd $path(tmp)} msg]} {
	puts $msg
	exit
    }
    if {![info exists path(db)]} {
	set path(db) $db(user).vit
    }
    if {[file pathtype $path(db)] == "relative"} {
	set path(db) $path(start)/$path(db)
    }
    if {![info exists path(out)]} {
	if {[file extension $path(db)] == ".vit"} {
	    set path(out) [file rootname $path(db)].$ctrl(target)
	} {
	    set path(out) [file tail $path(db)].$ctrl(target)
	}
    }
    if {[file pathtype $path(out)] == "relative"} {
	set path(out) $path(start)/$path(out)
    }
}

proc db_tmp_file_name {} {
    global path

    return $path(db).tmp
}

proc fill_all_windows {} {
    global var_list

    foreach t $var_list(types) {
	fill_window_info $t
    }
}

proc read_database {} {
    global db var_list path

    set name $path(db)
    if {![file exists $name]} {
	puts "Initializing \"$name\" as new database."
	fill_all_windows
	return
    }
    puts "Reading \"${name}\" as database."
    if {[catch {
	set db(reading) 1
	set dbfile [open $name r]
	gets $dbfile line
	if {[scan $line "# Resume Manager Database V%f" version] == 0} {
	    error "\"$name\" is not a vitae database file."
	}
	create_translation $version
	while {[gets $dbfile vname] >= 0} {
	    set vname [translate $vname]
	    switch -exact [winfo class $vname] {
		Entry {gets $dbfile db($vname)}
		Radiobutton {gets $dbfile db($vname)}
		Text {
	            gets $dbfile n_lines
	            gets $dbfile db($vname)
            	    for {set i 1} {$i < $n_lines} {incr i} {
			gets $dbfile a_line
			append db($vname) \n$a_line
		    }
		}
		Listbox {
	            gets $dbfile db($vname)
	            switch -regexp -- $vname {
			.education.deglist {
		            foreach i $db($vname) {
				create_degree $i
			    }
			}
			.experience.joblist {
		            foreach i $db($vname) {
				create_job $i
			    }
		        }
			.teachex.texlist {
		            foreach i $db($vname) {
				create_tex $i
			    }
		        }
			.references.reflist {
		            foreach i $db($vname) {
				create_reference $i
			    }
		        }
			.publications.pubtypelist {
		            foreach i $db($vname) {
				create_publication_type $i
			    }
		        }
			{.publication_type[0-9]*.publist} {
			    scan $vname ".publication_type%d" num
		            foreach i $db($vname) {
				create_publication${num}_ $i
			    }
		        }
		    }
	        }
	    }
	}
	close $dbfile
    } msg]} {
	puts $msg
	if {[info exists dbfile]} {
	    catch {close $dbfile}
	}
	puts "File \"${name}\" is not a valid vitae database."
	exit
    }

    fill_all_windows
    set db(reading) 0
}

proc save_database {} {
    global db var_list path

    # don't save intermediate states during a read
    if {[catch {if {$db(reading)} {error exit}}]} {return}

    set tmpname [db_tmp_file_name]
    set name    $path(db)
    if {[catch {
	set dbfile [open $tmpname w]
	puts $dbfile "# Resume Manager Database V1.03"
	foreach t $var_list(types) {
	    foreach ent $var_list($t) {
		if {$db($ent) != ""} {
		    puts $dbfile $ent
		    switch -exact [winfo class $ent] {
			Text {
        		    puts $dbfile [expr {[regsub -all \n \
				                 $db($ent) . trash] + 1}]
		            puts $dbfile $db($ent)
		        }
			default {puts $dbfile $db($ent)}
		    }
		}
	    }
	}
	close $dbfile
    } msg]} {
	catch {[close $dbfile]}
	puts "WARNING: FAILED TO SAVE CHANGES!\n   ($msg)"
	return
    }
    if {[catch {file rename -force $tmpname $name} msg]} {
	puts "save database: $msg"
    }
}
