proc add_vars list {
    global db var_list win_start

    set w [lindex $list 0]
    lappend var_list(types) $w
    set win_start($w) $w.[lindex $list 1]
    set var_list($w) ""
    foreach var [lrange $list 2 end] {
	if {[llength $var] < 2} {
	    lappend var_list($w) $w.$var
	    set db($w.$var) ""
	} else {
	    set v $w.[lindex $var 0]
	    lappend var_list($w) $v
	    set db($v) [lindex $var 1]
	}
    }
}

proc init_window_vars {} {
    global db var_list

    # initialize var_list
    set var_list(types) ""
    init_contact_vars
    init_objective_vars
    init_education_vars
    init_award_vars
    init_experience_vars
    init_teachex_vars
    init_activities_vars
    init_dissertation_vars
    init_publications_vars
    init_references_vars

    init_preview_vars
}

proc switch_windows tgt {
    global win_start

    wm deiconify $tgt
    raise $tgt
    tkTabToWindow $win_start($tgt)
}

proc create_main_window {} {
    global ctrl

    toplevel .main
    wm title .main "Vitae V1.03"

    set cf [frame .main.f]
    resm_button .main.generate -width 15 -text "Generate Vitae" \
	    -command create_resume
    resm_button .main.quit -width 15 -text "Exit Vitae" \
	    -command exit_vitae
    pack .main.generate -in $cf -side left -expand t -padx 5
    pack .main.quit -in $cf -side right -expand t -padx 5
    pack $cf -side bottom -pady 5 -expand t -fill x

    resm_button .main.contact -width 28 -text "Contact Information" \
	    -command {switch_windows .contact}
    resm_button .main.objective -width 28 -text "Objective and Interests" \
	    -command {switch_windows .objective}
    resm_button .main.education -width 28 -text "Education" \
	    -command {switch_windows .education}
    resm_button .main.award -width 28 -text "Honors, Awards, and Skills" \
	    -command {switch_windows .award}
    resm_button .main.experience -width 28 -text "Professional Experience" \
	    -command {switch_windows .experience}
    resm_button .main.teachex -width 28 -text "Teaching Experience" \
	    -command {switch_windows .teachex}
    resm_button .main.activities -width 28 -text "Activities" \
	    -command {switch_windows .activities}
    resm_button .main.dissertation -width 28 -text "Dissertation Overview" \
	    -command {switch_windows .dissertation}
    resm_button .main.publications -width 28 -text "Publications" \
	    -command {switch_windows .publications}
    resm_button .main.references -width 28 -text "References" \
	    -command {switch_windows .references}

    pack .main.contact .main.objective .main.education .main.award \
	    .main.experience .main.teachex .main.activities \
	    .main.dissertation .main.publications .main.references \
	    -side top -padx 5 -pady 2 -expand t

    wm resizable .main 0 0

    if {!$ctrl(use_gui)} {
	wm withdraw .main
    }
}

proc exit_vitae {} {
    global path

    if {[catch {cd $path(start)}]} {
	catch {cd /}
    }
    catch {file delete -force $path(tmp)}
    exit
}

proc create_change_window {win title {preview ""}} {
    toplevel $win
    wm title $win $title

    set f [frame $win.cf]
    resm_button $f.save -text "Save Changes" -command "save_window_info $win"
    pack $f.save -side left -padx 10 -pady 5
    resm_button $f.cancel -text "Cancel Changes" \
	    -command "fill_window_info $win"
    pack $f.cancel -side right -padx 10 -pady 5
    if {$preview != ""} {
	resm_button $f.preview -text "Save and Preview" \
		-command "preview_window_info $win $preview"
	pack $f.preview -padx 10 -pady 5
    }
    pack $f -side bottom -fill x

    return $win
}

proc create_close_window {win title {preview ""}} {
    toplevel $win
    wm title $win $title

    if {$preview == ""} {
	set b $win.close
	# need to save for publication list types
	resm_button $b -text "Close" -command "save_window_info $win"
	pack $b -side bottom -padx 5 -pady 5
    } {
	set f [frame $win.cf]
	resm_button $f.preview -text "Preview" \
		-command "preview_window_info $win $preview"
	pack $f.preview -side left -padx 10 -pady 5
	resm_button $f.close -text "Close" -command "save_window_info $win"
	pack $f.close -side right -padx 10 -pady 5
	pack $f -side bottom -fill x
    }
    return $win
}

proc fill_window_info win {
    global db var_list

    foreach ent $var_list($win) {
	switch -exact [winfo class $ent] {
	    Entry {
	        $ent delete 0 end
	        $ent insert 0 $db($ent)
            } \
            Text {
	        $ent delete 0.0 end
	        $ent insert 0.0 $db($ent)
            } \
	    Radiobutton {
	        set db(${ent}_tmp) $db($ent)
            } \
	    Listbox {
	        switch -regexp -- $ent {
                    .education.deglist {
		        $ent delete 0 end
		        foreach i $db($ent) {
			    $ent insert end $db(.degree${i}.deg)
			}
		    }
		    .experience.joblist {
		        $ent delete 0 end
		        foreach i $db($ent) {
			    $ent insert end $db(.job${i}.job)
			}
	            }
		    .teachex.texlist {
		        $ent delete 0 end
		        foreach i $db($ent) {
			    $ent insert end $db(.tex${i}.job)
			}
	            }
		    .references.reflist {
		        $ent delete 0 end
		        foreach i $db($ent) {
			    $ent insert end $db(.reference${i}.name)
			}
		    }
		    .publications.pubtypelist {
		        $ent delete 0 end
		        foreach i $db($ent) {
			    $ent insert end $db(.publication_type${i}.type)
			}
	            }
		    {.publication_type[0-9]*.publist} {
		        $ent delete 0 end
			scan $ent ".publication_type%d" num
		        foreach i $db($ent) {
			    $ent insert end $db(.publication${num}_${i}.name)
			}
	            }
		}
	    }
	}
    }

    wm withdraw $win
}

proc preview_window_info {win cmpt} {
    save_no_close $win
    create_component $cmpt
}

proc save_no_close win {
    global db var_list

    foreach ent $var_list($win) {
	switch -exact [winfo class $ent] {
	    Entry {
		# eliminate line breaks
		regsub -all \[\r\n\] [$ent get] " " db($ent)
	    }
	    Text  {
		set db($ent) \
			[string trimright [$ent get 0.0 "end -1 char"] \n]
	    }
	    Radiobutton {set db($ent) $db(${ent}_tmp)}
	}
    }
    save_database
}

proc save_window_info win {
    save_no_close $win
    wm withdraw $win
}

proc remove_traces var {
    upvar #0 $var rt
    set tlist [trace vinfo rt]
    foreach ti $tlist {
	trace vdelete rt [lindex $ti 0] [lindex $ti 1]
    }
}
