proc init_preview_vars {} {
    # placeholder to allow use of window construction procedures
    add_vars {
	.preview
	c
    }
}

proc create_preview_window {} {
    global preview

    set w [create_close_window .preview "Curriculum Vitae Section Preview"]

    set f [frame $w.cf]
    scrollbar $w.cys -command "$w.c yview "
    pack $w.cys -in $f -side right -fill y
    scrollbar $w.cxs -command "$w.c xview " -orient horizontal
    pack $w.cxs -in $f -side bottom -fill x
    canvas $w.c -width 100 -height 100 -highlightthickness 0 \
        -xscrollcommand "$w.cxs set" -yscrollcommand "$w.cys set" \
        -scrollregion {0 0 100 100}
    pack $w.c -in $f -expand t -fill both
    pack $f -expand t -fill both -padx 5 -pady 5

    wm withdraw $w

    set preview(image) ""
    set preview(elt)   ""
}


proc create_component type {
    global path ctrl preview

    # also fails if we lack convert (GhostScript)
    if {$ctrl(target) == "tex" || $ctrl(target) == "dvi"} {
	# should put up error dialog and say that preview is unavailable
	# should also differentiate between unavailable binaries
	# and final target being tex or dvi for another reason 
	# (such as user request)
	# return   <== not nice behavior; just fails silently
    }

    set tex_name [tex_file_name]
    set dvi_name [dvi_file_name]
    set eps_name [eps_file_name]
    set gif_name [gif_file_name]

    switch_windows .status
    tkwait visibility .status

    set_status "Building LaTeX preview file..." ""
    if {[catch {
	file copy -force $path(vitae_header) $tex_name
	set res_file [open $tex_name a]
	write_${type} $res_file
	puts $res_file "\\end{document}"
	close $res_file
    } msg]} {
	catch {close $res_file}
	set_status "Building LaTeX preview file..." "Failed: $msg"
	return
    }
    set_status "Building LaTeX file..." "Done."

    catch {file delete $dvi_name}
    if {![issue_command "$path(latex) $tex_name" "Latex'ing preview..."]} {
	return
    }
    catch {file delete $eps_name}
    if {![issue_command "$path(dvips) -E -o $eps_name $dvi_name" \
	    "Creating initial EPS file..."]} {
	return
    }

    set_status "Finding Bounding Box..." ""
    if {[catch {
	set f [open $eps_name r]
	while {1} {
	    if {[gets $f line] == -1} {
		error "no bounding box found."
	    }
	    if {[lindex $line 0] == "%%BoundingBox:"} {
		set x1 [lindex $line 1]
		set y1 [lindex $line 2]
		set wd [expr {[lindex $line 3] - $x1 + 1}]
		set ht [expr {[lindex $line 4] - $y1 + 1}]
		break
	    }
	}
	close $f
    } msg]} {
	catch {close $f}
	set_status "Finding Bounding Box..." "Failed: $msg"
	return
    }

    if {![issue_command \
          "$path(convert) $path(convert-fix) $eps_name $gif_name" \
	  "Translating to GIF..."]} {
	puts foo
	return
    }


    set w .preview
    if {$preview(elt) != ""} {
	image delete $preview(elt)
	$w.c delete $preview(image)
    }
    set preview(elt) [image create photo -format gif -file $gif_name \
	              -height $ht -width $wd]
    set preview(image) [$w.c create image 0 0 -anchor nw -image $preview(elt)]
    $w.c config -height $ht -width $wd
    $w.c config -scrollregion "0 0 $wd $ht"
    $w.c xview moveto 0
    $w.c yview moveto 0

    set_status "Preview generation is complete." ""
    update
    switch_windows $w
}
