proc create_status_window {} {
    global status win_start

    set w .status
    toplevel $w
    wm title $w "Generation Status"

    set f [frame $w.bf]
    resm_button $f.clear -text "Clear" -command clear_status
    pack $f.clear -side left -padx 10 -pady 5
    resm_button $f.close -text "Close" -command close_status
    pack $f.close -side right -padx 10 -pady 5
    resm_button $f.cancel -text "Cancel" -command cancel_command
    pack $f.cancel -padx 10 -pady 5
    pack $f -side bottom -fill x

    set status(text) ""
    label $w.state -textvariable status(text)
    pack $w.state -side bottom -fill x

    set status(op) ""
    label $w.op -textvariable status(op)
    pack $w.op -side bottom -fill x

    set f [frame $w.tf]
    scrollbar $f.s -command "$w.out yview"
    pack $f.s -side right -fill y
    resm_text $w.out -height 40 -width 80 -state disabled \
	    -yscrollcommand "$f.s set"
    pack $w.out -in $f -expand t -fill both
    pack $f -expand t -fill both -padx 5 -pady 5

    wm withdraw $w

    set win_start($w) $w.out

    set status(file) ""
}

proc clear_status {} {
    set w .status.out
    $w config -state normal
    $w delete 0.0 end
    $w config -state disabled
}

proc end_status_line {} {
    set w .status.out
    if {[$w index "end -1 char"] != [$w index "end -1 char linestart"]} {
	$w config -state normal
	$w insert end \n
	$w see end
	$w config -state disabled
    }
}

proc append_status txt {
    set w .status.out
    $w config -state normal
    $w insert end $txt
    $w see end
    $w config -state disabled
}

proc set_clock_status {} {
    global status

    if {$status(clock) == 0} {
	set status(text) "Operation timed out."
    } {
	set status(text) "Timeout in [expr {$status(clock) / 10}].[expr \
		{$status(clock) % 10}] seconds."
    }
}

proc start_clock {} {
    global status

    set status(clock) 150
    set_clock_status
    set status(after) [after 100 timeout_countdown]
}

proc timeout_countdown {} {
    global status

    if {$status(command) != "pending"} {
	return
    }
    incr status(clock) -1
    set_clock_status
    if {$status(clock) == 0} {
	halt_command
	set status(command) timeout
    } {
	set status(after) [after 100 timeout_countdown]
    }
}

proc issue_command {cmd title} {
    global status ctrl

    if {$ctrl(use_gui)} {
	switch_windows .status
    }
    if {![info exists status(command)]} {
	set status(command) complete
    }
    if {$status(command) == "pending"} {
	raise .status
	return 0
    }

    set status(op) $title

    if {[catch {
	set f [open "|$cmd" r+]
	fconfigure $f -blocking 0
	set status(file) $f
    } msg]} {
	catch {close $f}
	set status(text) $msg
	set status(file) ""
	return 0
    }
    
    set status(command) pending
    append_status --------------------------------------------------------------------------------\n
    append_status $cmd\n
    append_status --------------------------------------------------------------------------------\n
    start_clock
    fileevent $f readable read_output
    tkwait variable status(command)
    if {$status(command) != "complete"} {
	return 0
    }
    return 1
}

proc cancel_command {} {
    global status

    catch {after cancel $status(after)}
    set status(text) "Operation cancelled."
    halt_command
    set status(command) cancelled
}

proc close_status {} {
    cancel_command
    wm withdraw .status
}

proc read_output {} {
    global status

    set f $status(file)
    if {![eof $f]} {
	set line [read $f 16384]
	append_status ${line}
	return
    }
    halt_command
    set status(text) Done.
    set status(command) complete
}

proc halt_command {} {
    global status

    if {[set f $status(file)] == ""} {
	return
    }
    if {[catch {close $f} errout]} {
	end_status_line
	append_status ${errout}\n\n\n
    } {
	append_status \n\n
    }
    set status(file) ""
}

proc set_status {op msg} {
    global status

    set status(op)   $op
    set status(text) $msg
}
