proc resm_entry path {
    entry $path -highlightthickness 0 -width 50
}

proc resm_radio {path wdth txt var} {
    radiobutton $path -highlightthickness 0 -text $txt -variable $var \
	    -value $txt -width $wdth -anchor w
}

proc resm_text {path args} {
    eval "text $path -wrap word -width 55 -height 5 -highlightthickness 0 \
	    $args"
}

proc resm_list path {
    listbox $path -width 40 -height 10 -highlightthickness 0 -selectmode browse
}

proc resm_button {path args} {
    eval "button $path -highlightthickness 0 $args"
}

proc make_frames {var prefix num} {
    
    if {[llength $var] == 1} {
	upvar $var v
	set v ${prefix}_0
    } {
	set i 0
	foreach vname $var {
	    upvar $vname v
	    set v ${prefix}_${i}
	    incr i
	}
    }
    for {set i 0} {$i < $num} {incr i} {
	frame ${prefix}_${i}
    }
}

proc use_next_frame var {
    upvar $var v
    set where [string last _ $v]
    set i [string range $v [expr {$where + 1}] end]
    incr i
    set v [string range $v 0 $where]${i}
}

proc create_form {frame prefix info} {
    # Make two frames starting with "${frame}.f" and set variables left and
    # right to their names.
    make_frames {left right} ${frame}.f 2 

    foreach item $info {
	set name ${prefix}.[lindex $item 2]
	label ${name}_l -text [lindex $item 0] -anchor ne
	eval "[concat [lindex $item 1] ${name} [lrange $item 3 end]]"
	switch -exact [winfo class ${name}] {
	    Text {
		pack ${name}_l -in ${left} -side top -expand t -fill both
		pack ${name} -in ${right} -side top -expand t -fill both
	    }
	    default {
		pack ${name}_l -in ${left} -side top -fill x
		pack ${name} -in ${right} -side top -fill x
	    }
	}
    }

    pack ${left} -in ${frame} -side left -fill y
    pack ${right} -in ${frame} -side right -expand t -fill both
}

proc create_radio_group {frame prefix info {max 1}} {
    ${frame} config -borderwidth 2 -relief ridge
    set i 0
    foreach elt [lrange $info 0 end] {
	set rb ${prefix}_$i
	resm_radio $rb $max $elt db(${prefix}_0_tmp)
	pack $rb -in $frame -side top -anchor w
	if {[set width [expr {[string length $elt] + 1}]] > $max} {
	    set max $width
	}
	incr i
    }
    for {} {$i > 0} {} {
	incr i -1
	${prefix}_$i config -width $max
    }
}

proc create_controlled_list {frame prefix lname opname tvar title} {

    set l $prefix.$lname
    resm_list $l
    pack $l -in $frame -side left -expand t -fill both -padx 5 -pady 5
    bind $l <Return> edit_${opname}

    resm_button $prefix.new -text "New" -width 10 -command new_${opname}
    resm_button $prefix.up -text "Move Up" -width 10 -command move_${opname}_up
    resm_button $prefix.down -text "Move Down" -width 10 \
	    -command move_${opname}_down
    resm_button $prefix.edit -text "Edit" -width 10 -command edit_${opname}
    resm_button $prefix.del  -text "Delete" -width 10 -command delete_${opname}

    set f [frame $frame.f]
    pack $prefix.new $prefix.up $prefix.down $prefix.edit $prefix.del \
	    -in $f -side top -padx 5 -pady 5
    pack $f -side right

    set dbentry "\$db(.${opname}\${num}.$tvar)"
    proc save_${opname}_name {num v1 v2 op} "
        global db

	set i \[lsearch \$db($l) \$num\]
	$l delete \$i
	$l insert \$i \$db(.${opname}\${num}.$tvar)
	$l selection clear 0 end
	$l selection set \$i

        wm title .${opname}\${num} \"[subst $title]\"
    "

    proc new_${opname} {} "
	global db

	for {set i 0} {\[lsearch \$db($l) \$i\] != -1 && \$i < 10} {incr i} {}
	if {\$i == 10} {
	    return
	}
	create_${opname} \$i
	lappend db($l) \$i
	$l insert end \$db(.${opname}\${i}.$tvar)
	
	fill_window_info .${opname}\${i}
	# perform packing before showing window outline to user
	update
	switch_windows .${opname}\${i}

	save_database
    "

    proc swap_${opname} {p1 p2} "
	global db
    
	set i1 \[lindex \$db($l) \$p1\]
        set i2 \[lindex \$db($l) \$p2\]
        set db($l) \[lreplace \$db($l) \$p1 \$p2 \$i2 \$i1]

        # trace will update list in education window
        set db(.${opname}\${i1}.$tvar) \$db(.${opname}\${i1}.$tvar)
        set db(.${opname}\${i2}.$tvar) \$db(.${opname}\${i2}.$tvar)

        save_database
    "

    proc move_${opname}_up {} "
	global db

        if {\[set pos \[$l curselection\]\] == {}} {
	    return
	}
	if {\$pos == 0} {
	    return
	}
	set prev \[expr {\$pos - 1}\]
	swap_${opname} \$prev \$pos
	$l selection clear 0 end
	$l selection set \$prev
    "

    proc move_${opname}_down {} "
	global db
	
	if {\[set pos \[$l curselection\]\] == {}} {
	    return
	}
	if {\$pos == \[expr {\[$l size\] - 1}\]} {
	    return
	}
	set next \[expr {\$pos + 1}\]
	swap_${opname} \$pos \$next
	$l selection clear 0 end
	$l selection set \$next
    "

    proc edit_${opname} {} "
	global db

	if {\[set pos \[$l curselection\]\] == {}} {
	    return
	}
	set i \[lindex \$db($l) \$pos\]
	switch_windows .${opname}\${i}
    "
    
    proc delete_${opname} {} "
	global db var_list win_start

	if {\[set pos \[$l curselection\]\] == {}} {
	    return
	}
	set i \[lindex \$db($l) \$pos\]
	set db($l) \[lreplace \$db($l) \$pos \$pos\]
	set vln \[lsearch \$var_list(types) .${opname}\${i}\]
	set var_list(types) \[lreplace \$var_list(types) \$vln \$vln\]
	destroy .${opname}\${i}
	remove_traces db(.${opname}\${i}.$tvar)
	$l delete \$pos
	save_database
    "
}
    
proc motion_link {top bottom bot_prev} {
    if {[winfo class $top] == "Entry"} {
	bind $top <Control-p> "
	    tkTabToWindow $bottom
	    break
	"
    }
    bind $top <Shift-Tab> "
	tkTabToWindow $bottom
	break
    "
    # Unfortunately, not all old Tcl/Tk's recognize this event.
    catch {bind $top <ISO_Left_Tab> "
	tkTabToWindow $bottom
	break
    "}
    bind $bottom <Tab> "
	tkTabToWindow $top
	break
    "
    # need to prevent Tab binding from usurping Shift-Tab's role
    bind $bottom <Shift-Tab> "
	tkTabToWindow $bot_prev
	break
    "
    if {[winfo class $bottom] == "Entry"} {
	bind $bottom <Control-n> "
	    tkTabToWindow $top
	    break
        "
	bind $bottom <Return> "
	    tkTabToWindow $top
	    break
        "
    }
}

# call this after motion_link
proc save_on_return w {
    bind $w <Return> "
	save_window_info [winfo toplevel $w]
	break
    "
}

proc text_tabs {w prev next} {
    bind $w <Shift-Tab> "
	tkTabToWindow $prev
	break
    "
    # Unfortunately, not all old Tcl/Tk's recognize this event.
    catch {bind $w <ISO_Left_Tab> "
	tkTabToWindow $prev
	break
    "}
    bind $w <Tab> "
	tkTabToWindow $next
	break
    "
}

proc fix_compatibility {} {
    bind Entry <Control-p> {tkTabToWindow [tk_focusPrev %W]}
    bind Entry <Control-n> {tkTabToWindow [tk_focusNext %W]}
    bind Entry <Return>    {tkTabToWindow [tk_focusNext %W]}

    # Linux compatibility
    # Unfortunately, not all old Tcl/Tk's recognize this event.
    catch {bind all <ISO_Left_Tab> {focus [tk_focusPrev %W]}}

    if {[info command bind] == ""} {puts failed}

    # Tk compatibility (from Tk8.0)
    if {[info command tkTabToWindow] == ""} {
	proc tkTabToWindow {w} {
	    if {"[winfo class $w]" == "Entry"} {
		$w select range 0 end
		$w icur end
	    }
	    focus $w
	}
	
	bind Entry <Tab> {tkTabToWindow [tk_focusNext %W]}
	bind Entry <Shift-Tab> {tkTabToWindow [tk_focusNext %W]}
	# Unfortunately, not all old Tcl/Tk's recognize this event.
	catch {bind Entry <ISO_Left_Tab> {tkTabToWindow [tk_focusPrev %W]}}
    }
}
