proc create_translation {version} {
    global translate

    switch -exact -- $version {
	"1.03" {
	    set translate(on) 0
	    return
	}
	"1.02" {
	    puts "Translating database from version 1.02."
	    puts "Database will be saved as version 1.03."
	    set translate(on) 0
	    return
	}
	"1.01" {
	    puts "Translating database from version 1.01."
	    puts "Database will be saved as version 1.03."
	    set translate(on) 0
	    return
	}
	"1.0" {
	    puts "Translating database from version 1.0."
	    puts "Database will be saved as version 1.03."
	    set translate(on) 1
	}
	default {
	    error "Database translation from version $version not supported."
	}
    }

    foreach pair {
	{activities.r.act .activities.act_0}
	{activities.t .activities.text}
	{award.r.awa .award.award_0}
	{award.t .award.award_text}
	{resskills.r.ts .award.res_skl_0}
	{resskills.t .award.res_skl_text}
	{tchskills.r.tch .award.tch_skl_0}
	{tchskills.t .award.tch_sklitext}
	{name.e .contact.name}
	{home.e.addr1 .contact.home_addr1}
	{home.e.addr2 .contact.home_addr2}
	{home.e.addr3 .contact.home_addr3}
	{home.e.phone .contact.home_phone}
	{work.e.addr1 .contact.work_addr1}
	{work.e.addr2 .contact.work_addr2}
	{work.e.addr3 .contact.work_addr3}
	{work.e.phone .contact.work_phone}
	{work.e.fax .contact.work_fax}
	{elec.e.email .contact.email}
	{elec.e.url .contact.home_url}
	{elec.e.proj .contact.proj_url}
	{distitle.e .dissertation.title}
	{dissertation.r.dis .dissertation.dis_0}
	{dissertation.t .dissertation.text}
	{deglist .education.deglist}
	{r.pe .experience.exp_0}
	{joblist .experience.joblist}
	{objective.r.soo .objective.obj_0}
	{objective.t .objective.obj_text}
	{resintr.r.ri .objective.res_int_0}
	{resintr.t .objective.res_int_text}
	{tchintr.r.ti .objective.tch_int_0}
	{tchintr.t .objective.tch_int_text}
	{pubtypelist .publications.pubtypelist}
	{reflist .references.reflist}
    } {
	set translate(fixed,[lindex $pair 0]) [lindex $pair 1]
    }

    set translate(subst) {
	{deg([0-9]*).e.inst .degree${i}.inst}
	{deg([0-9]*).e.loc .degree${i}.loc}
	{deg([0-9]*).e.deg .degree${i}.deg}
	{deg([0-9]*).e.fld .degree${i}.fld}
	{deg([0-9]*).e.r.deg_when([0-9]*) .degree${i}.stat.a}
	{deg([0-9]*).e.date .degree${i}.date}
	{deg([0-9]*).e.adv .degree${i}.adv}
	{deg([0-9]*).e.tit .degree${i}.tit}
	{deg([0-9]*).e.desc .degree${i}.desc}
	{deg([0-9]*).e.gpa .degree${i}.gpa}
	{deg([0-9]*).e.rank .degree${i}.rank}
	{job([0-9]*).e.inst  .job${i}.inst}
	{job([0-9]*).e.loc   .job${i}.loc}
	{job([0-9]*).e.job .job${i}.job}
	{job([0-9]*).e.date  .job${i}.date}
	{job([0-9]*).e.desc .job${i}.desc}
	{ptype([0-9]*).e .publication_type${i}.type}
	{publist([0-9]*) .publication_type${i}.publist}
	{ref([0-9]*).e.name .reference${i}.name}
	{ref([0-9]*).e.posn .reference${i}.posn}
	{ref([0-9]*).e.inst  .reference${i}.inst}
	{ref([0-9]*).e.addr1 .reference${i}.addr1}
	{ref([0-9]*).e.addr2 .reference${i}.addr2}
	{ref([0-9]*).e.addr3 .reference${i}.addr3}
	{ref([0-9]*).e.addr4 .reference${i}.addr4}
	{ref([0-9]*).e.phone .reference${i}.phone}
	{ref([0-9]*).e.email .reference${i}.email}
	{pub([0-9]*)_([0-9]*).e.name .publication${i}_${j}.name}
	{pub([0-9]*)_([0-9]*).e.desc .publication${i}_${j}.desc}
    }
}

proc translate {var} {
    global db translate

    if {!$translate(on)} {
	return $var
    }
    if {[info exists translate(fixed,$var)]} {
	return $translate(fixed,$var)
    }
    set i ""
    set j ""
    foreach pair $translate(subst) {
	if {[regexp [lindex $pair 0] $var trash i j]} {
	    return [subst [lindex $pair 1]]
	}
    }
    error "No translation exists for variable $var."
}
